@@ -51,7 +51,7 @@ from pcluster.api.models import (
     UpdateError,
     ValidationLevel,
 )
-from pcluster.api.util import assert_node_executable
+from pcluster.api.util import assert_valid_node_js
 from pcluster.aws.aws_api import AWSApi
 from pcluster.aws.common import StackNotFoundError
 from pcluster.config.update_policy import UpdatePolicy
@@ -97,7 +97,7 @@ def create_cluster(
     (Defaults to &#39;true&#39;.)
     :type rollback_on_failure: bool
     """
-    assert_node_executable()
+    assert_valid_node_js()
     # Set defaults
     configure_aws_region_from_config(region, create_cluster_request_content["clusterConfiguration"])
     rollback_on_failure = rollback_on_failure in {True, None}
@@ -116,7 +116,7 @@ def create_cluster(
 
         if dryrun:
             ignored_validation_failures = cluster.validate_create_request(
-                get_validator_suppressors(suppress_validators), FailureLevel[validation_failure_level]
+                get_validator_suppressors(suppress_validators), FailureLevel[validation_failure_level], dry_run=dryrun
             )
             validation_messages = validation_results_to_config_validation_errors(ignored_validation_failures)
             raise DryrunOperationException(validation_messages=validation_messages or None)
@@ -316,7 +316,7 @@ def update_cluster(
 
     :rtype: UpdateClusterResponseContent
     """
-    assert_node_executable()
+    assert_valid_node_js()
     # Set defaults
     configure_aws_region_from_config(region, update_cluster_request_content["clusterConfiguration"])
     validation_failure_level = validation_failure_level or ValidationLevel.ERROR
@@ -343,6 +343,7 @@ def update_cluster(
                 force=force_update,
                 validator_suppressors=get_validator_suppressors(suppress_validators),
                 validation_failure_level=FailureLevel[validation_failure_level],
+                dry_run=dryrun,
             )
             change_set, _ = _analyze_changes(changes)
             validation_messages = validation_results_to_config_validation_errors(ignored_validation_failures)
