@@ -14,7 +14,7 @@
 #
 import logging
 from enum import Enum
-from typing import List, Union
+from typing import Dict, List, Union
 
 import pkg_resources
 
@@ -32,9 +32,18 @@ from pcluster.constants import (
     EBS_VOLUME_SIZE_DEFAULT,
     EBS_VOLUME_TYPE_DEFAULT,
     EBS_VOLUME_TYPE_IOPS_DEFAULT,
+    MAX_NUMBER_OF_COMPUTE_RESOURCES,
+    MAX_NUMBER_OF_QUEUES,
     MAX_STORAGE_COUNT,
+    SUPPORTED_OSES,
+)
+from pcluster.utils import (
+    get_attr,
+    get_installed_version,
+    get_partition,
+    get_resource_name_from_resource_arn,
+    replace_url_parameters,
 )
-from pcluster.utils import get_partition, get_resource_name_from_resource_arn
 from pcluster.validators.awsbatch_validators import (
     AwsBatchComputeInstanceTypeValidator,
     AwsBatchComputeResourceSizeValidator,
@@ -49,10 +58,8 @@ from pcluster.validators.cluster_validators import (
     CustomAmiTagValidator,
     DcvValidator,
     DisableSimultaneousMultithreadingArchitectureValidator,
-    DuplicateInstanceTypeValidator,
     DuplicateMountDirValidator,
     DuplicateNameValidator,
-    EfaGdrValidator,
     EfaOsArchitectureValidator,
     EfaPlacementGroupValidator,
     EfaSecurityGroupValidator,
@@ -66,6 +73,8 @@ from pcluster.validators.cluster_validators import (
     InstanceArchitectureCompatibilityValidator,
     IntelHpcArchitectureValidator,
     IntelHpcOsValidator,
+    MaxCountValidator,
+    MixedSecurityGroupOverwriteValidator,
     NameValidator,
     NumberOfStorageValidator,
     OverlappingMountDirValidator,
@@ -73,6 +82,7 @@ from pcluster.validators.cluster_validators import (
     SchedulerOsValidator,
     SharedStorageNameValidator,
 )
+from pcluster.validators.directory_service_validators import DomainAddrValidator, LdapTlsReqCertValidator
 from pcluster.validators.ebs_validators import (
     EbsVolumeIopsValidator,
     EbsVolumeSizeSnapshotValidator,
@@ -107,6 +117,12 @@ from pcluster.validators.s3_validators import (
     S3BucketValidator,
     UrlValidator,
 )
+from pcluster.validators.scheduler_plugin_validators import (
+    SchedulerPluginOsArchitectureValidator,
+    SchedulerPluginRegionValidator,
+    SudoPrivilegesValidator,
+    SupportedVersionsValidator,
+)
 
 LOGGER = logging.getLogger(__name__)
 
@@ -462,6 +478,12 @@ class AwsBatchQueueNetworking(_QueueNetworking):
         super().__init__(**kwargs)
 
 
+class SchedulerPluginQueueNetworking(SlurmQueueNetworking):
+    """Represent the networking configuration for the Scheduler Plugin Queue."""
+
+    pass
+
+
 class Ssh(Resource):
     """Represent the SSH configuration for a node."""
 
@@ -490,10 +512,7 @@ class Efa(Resource):
     def __init__(self, enabled: bool = None, gdr_support: bool = None, **kwargs):
         super().__init__(**kwargs)
         self.enabled = enabled
-        self.gdr_support = gdr_support
-
-    def _register_validators(self):
-        self._register_validator(EfaGdrValidator, gdr_support=self.gdr_support)
+        self.gdr_support = Resource.init_param(gdr_support, default=False)
 
 
 # ---------------------- Monitoring ---------------------- #
@@ -650,6 +669,42 @@ class Imds(Resource):
         self.secured = Resource.init_param(secured, default=True)
 
 
+class DirectoryService(Resource):
+    """Represent DirectoryService configuration."""
+
+    def __init__(
+        self,
+        domain_name: str = None,
+        domain_addr: str = None,
+        password_secret_arn: str = None,
+        domain_read_only_user: str = None,
+        ldap_tls_ca_cert: str = None,
+        ldap_tls_req_cert: str = None,
+        ldap_access_filter: str = None,
+        generate_ssh_keys_for_users: bool = None,
+        additional_sssd_configs: Dict = None,
+        **kwargs,
+    ):
+        super().__init__(**kwargs)
+        self.domain_name = Resource.init_param(domain_name)
+        self.domain_addr = Resource.init_param(domain_addr)
+        self.password_secret_arn = Resource.init_param(password_secret_arn)
+        self.domain_read_only_user = Resource.init_param(domain_read_only_user)
+        self.ldap_tls_ca_cert = Resource.init_param(ldap_tls_ca_cert)
+        self.ldap_tls_req_cert = Resource.init_param(ldap_tls_req_cert, default="hard")
+        self.ldap_access_filter = Resource.init_param(ldap_access_filter)
+        self.generate_ssh_keys_for_users = Resource.init_param(generate_ssh_keys_for_users, default=True)
+        self.additional_sssd_configs = Resource.init_param(additional_sssd_configs, default={})
+
+    def _register_validators(self):
+        if self.domain_addr:
+            self._register_validator(
+                DomainAddrValidator, domain_addr=self.domain_addr, additional_sssd_configs=self.additional_sssd_configs
+            )
+        if self.ldap_tls_req_cert:
+            self._register_validator(LdapTlsReqCertValidator, ldap_tls_reqcert=self.ldap_tls_req_cert)
+
+
 class ClusterIam(Resource):
     """Represent the IAM configuration for Cluster."""
 
@@ -921,6 +976,7 @@ class BaseClusterConfig(Resource):
         additional_packages: AdditionalPackages = None,
         tags: List[Tag] = None,
         iam: ClusterIam = None,
+        directory_service: DirectoryService = None,
         config_region: str = None,
         custom_s3_bucket: str = None,
         additional_resources: str = None,
@@ -942,6 +998,7 @@ class BaseClusterConfig(Resource):
         self.additional_packages = additional_packages
         self.tags = tags
         self.iam = iam
+        self.directory_service = directory_service
         self.custom_s3_bucket = Resource.init_param(custom_s3_bucket)
         self._bucket = None
         self.additional_resources = Resource.init_param(additional_resources)
@@ -977,9 +1034,11 @@ class BaseClusterConfig(Resource):
         )
         self._register_storage_validators()
         self._register_validator(
-            HeadNodeLaunchTemplateValidator, head_node=self.head_node, ami_id=self.head_node_ami, tags=self.tags
+            HeadNodeLaunchTemplateValidator,
+            head_node=self.head_node,
+            ami_id=self.head_node_ami,
+            tags=self.get_cluster_tags(),
         )
-
         if self.head_node.dcv:
             self._register_validator(
                 DcvValidator,
@@ -1191,6 +1250,10 @@ class BaseClusterConfig(Resource):
             )
         return self._official_ami
 
+    def get_cluster_tags(self):
+        """Return tags configured in the cluster configuration."""
+        return self.tags
+
 
 class AwsBatchComputeResource(BaseComputeResource):
     """Represent the AwsBatch Compute Resource."""
@@ -1333,6 +1396,7 @@ class SlurmComputeResource(BaseComputeResource):
             EfaValidator,
             instance_type=self.instance_type,
             efa_enabled=self.efa.enabled,
+            gdr_support=self.efa.gdr_support,
         )
 
     @property
@@ -1384,13 +1448,11 @@ class SlurmComputeResource(BaseComputeResource):
         return self.disable_simultaneous_multithreading and not self.instance_type_info.is_cpu_options_supported_in_lt()
 
 
-class SlurmQueue(BaseQueue):
-    """Represent the Slurm Queue resource."""
+class _CommonQueue(BaseQueue):
+    """Represent the Common Queue resource between Slurm and Scheduler Plugin."""
 
     def __init__(
         self,
-        compute_resources: List[SlurmComputeResource],
-        networking: SlurmQueueNetworking,
         compute_settings: ComputeSettings = None,
         custom_actions: CustomActions = None,
         iam: Iam = None,
@@ -1398,16 +1460,151 @@ class SlurmQueue(BaseQueue):
         **kwargs,
     ):
         super().__init__(**kwargs)
-        self.compute_resources = compute_resources
-        self.networking = networking
         self.compute_settings = compute_settings or ComputeSettings(implied=True)
         self.custom_actions = custom_actions
         self.iam = iam or Iam(implied=True)
         self.image = image
 
+    @property
+    def instance_role(self):
+        """Return the IAM role for compute nodes, if set."""
+        return self.iam.instance_role if self.iam else None
+
+    @property
+    def instance_profile(self):
+        """Return the IAM instance profile for compute nodes, if set."""
+        return self.iam.instance_profile if self.iam else None
+
+    @property
+    def queue_ami(self):
+        """Return queue image id."""
+        if self.image and self.image.custom_ami:
+            return self.image.custom_ami
+        else:
+            return None
+
+
+class SlurmQueue(_CommonQueue):
+    """Represent the Slurm Queue resource."""
+
+    def __init__(
+        self,
+        compute_resources: List[SlurmComputeResource],
+        networking: SlurmQueueNetworking,
+        **kwargs,
+    ):
+        super().__init__(**kwargs)
+        self.compute_resources = compute_resources
+        self.networking = networking
+
+    def _register_validators(self):
+        super()._register_validators()
+        self._register_validator(
+            DuplicateNameValidator,
+            name_list=[compute_resource.name for compute_resource in self.compute_resources],
+            resource_name="Compute resource",
+        )
+        self._register_validator(
+            MaxCountValidator,
+            resources_length=len(self.compute_resources),
+            max_length=MAX_NUMBER_OF_COMPUTE_RESOURCES,
+            resource_name="ComputeResources",
+        )
+        for compute_resource in self.compute_resources:
+            self._register_validator(
+                CapacityTypeValidator, capacity_type=self.capacity_type, instance_type=compute_resource.instance_type
+            )
+            self._register_validator(
+                EfaSecurityGroupValidator,
+                efa_enabled=compute_resource.efa.enabled,
+                security_groups=self.networking.security_groups,
+                additional_security_groups=self.networking.additional_security_groups,
+            )
+            self._register_validator(
+                EfaPlacementGroupValidator,
+                efa_enabled=compute_resource.efa.enabled,
+                placement_group_enabled=self.networking.placement_group and self.networking.placement_group.enabled,
+                placement_group_config_implicit=self.networking.placement_group is None
+                or self.networking.placement_group.is_implied("enabled"),
+            )
+
+    @property
+    def instance_type_list(self):
+        """Return the list of instance types associated to the Queue."""
+        return [compute_resource.instance_type for compute_resource in self.compute_resources]
+
+
+class Dns(Resource):
+    """Represent the DNS settings."""
+
+    def __init__(
+        self, disable_managed_dns: bool = None, hosted_zone_id: str = None, use_ec2_hostnames: bool = None, **kwargs
+    ):
+        super().__init__(**kwargs)
+        self.disable_managed_dns = Resource.init_param(disable_managed_dns, default=False)
+        self.hosted_zone_id = Resource.init_param(hosted_zone_id)
+        self.use_ec2_hostnames = Resource.init_param(use_ec2_hostnames, default=False)
+
+
+class SlurmSettings(Resource):
+    """Represent the Slurm settings."""
+
+    def __init__(self, scaledown_idletime: int = None, dns: Dns = None, **kwargs):
+        super().__init__()
+        self.scaledown_idletime = Resource.init_param(scaledown_idletime, default=10)
+        self.dns = dns or Dns(implied=True)
+
+
+class SlurmScheduling(Resource):
+    """Represent a slurm Scheduling resource."""
+
+    def __init__(self, queues: List[SlurmQueue], settings: SlurmSettings = None):
+        super().__init__()
+        self.scheduler = "slurm"
+        self.queues = queues
+        self.settings = settings or SlurmSettings(implied=True)
+
+    def _register_validators(self):
+        self._register_validator(
+            DuplicateNameValidator, name_list=[queue.name for queue in self.queues], resource_name="Queue"
+        )
+        self._register_validator(
+            MaxCountValidator,
+            resources_length=len(self.queues),
+            max_length=MAX_NUMBER_OF_QUEUES,
+            resource_name="SlurmQueues",
+        )
+
+
+class SchedulerPluginComputeResource(SlurmComputeResource):
+    """Represent the Scheduler Plugin Compute Resource."""
+
+    def __init__(
+        self,
+        custom_settings: Dict = None,
+        **kwargs,
+    ):
+        super().__init__(**kwargs)
+        self.custom_settings = custom_settings
+
+
+class SchedulerPluginQueue(_CommonQueue):
+    """Represent the Scheduler Plugin queue."""
+
+    def __init__(
+        self,
+        compute_resources: List[SchedulerPluginComputeResource],
+        networking: SchedulerPluginQueueNetworking,
+        custom_settings: Dict = None,
+        **kwargs,
+    ):
+        super().__init__(**kwargs)
+        self.compute_resources = compute_resources
+        self.networking = networking
+        self.custom_settings = custom_settings
+
     def _register_validators(self):
         super()._register_validators()
-        self._register_validator(DuplicateInstanceTypeValidator, instance_type_list=self.instance_type_list)
         self._register_validator(
             DuplicateNameValidator,
             name_list=[compute_resource.name for compute_resource in self.compute_resources],
@@ -1436,62 +1633,277 @@ class SlurmQueue(BaseQueue):
         """Return the list of instance types associated to the Queue."""
         return [compute_resource.instance_type for compute_resource in self.compute_resources]
 
-    @property
-    def instance_role(self):
-        """Return the IAM role for compute nodes, if set."""
-        return self.iam.instance_role if self.iam else None
 
-    @property
-    def instance_profile(self):
-        """Return the IAM instance profile for compute nodes, if set."""
-        return self.iam.instance_profile if self.iam else None
+class SchedulerPluginSupportedDistros(Resource):
+    """Represent the Supported Distros for a Scheduler Plugin."""
 
-    @property
-    def queue_ami(self):
-        """Return queue image id."""
-        if self.image and self.image.custom_ami:
-            return self.image.custom_ami
-        else:
-            return None
-
-
-class Dns(Resource):
-    """Represent the DNS settings."""
-
-    def __init__(self, disable_managed_dns: bool = None, hosted_zone_id: str = None):
-        super().__init__()
-        self.disable_managed_dns = Resource.init_param(disable_managed_dns, default=False)
-        self.hosted_zone_id = Resource.init_param(hosted_zone_id)
-
-
-class SlurmSettings(Resource):
-    """Represent the Slurm settings."""
-
-    def __init__(self, scaledown_idletime: int = None, dns: Dns = None, **kwargs):
+    def __init__(self, x86: List[str] = None, arm64: List[str] = None, **kwargs):
         super().__init__(**kwargs)
-        self.scaledown_idletime = Resource.init_param(scaledown_idletime, default=10)
-        self.dns = dns
+        self.x86 = Resource.init_param(x86, default=SUPPORTED_OSES)
+        self.arm64 = Resource.init_param(arm64, default=SUPPORTED_OSES)
 
 
-class SlurmScheduling(Resource):
-    """Represent a slurm Scheduling resource."""
+class SchedulerPluginQueueConstraints(Resource):
+    """Represent the Queue Constraints for a Scheduler Plugin."""
 
-    def __init__(self, queues: List[SlurmQueue], settings: SlurmSettings = None):
-        super().__init__()
-        self.scheduler = "slurm"
+    def __init__(self, max_count: int = None, **kwargs):
+        super().__init__(**kwargs)
+        self.max_count = Resource.init_param(max_count, default=MAX_NUMBER_OF_QUEUES)
+
+
+class SchedulerPluginComputeResourceConstraints(Resource):
+    """Represent the Compute Resource Constraints for a Scheduler Plugin."""
+
+    def __init__(self, max_count: int = None, **kwargs):
+        super().__init__(**kwargs)
+        self.max_count = Resource.init_param(max_count, default=MAX_NUMBER_OF_COMPUTE_RESOURCES)
+
+
+class SchedulerPluginRequirements(Resource):
+    """Represent the Requirements for a Scheduler Plugin."""
+
+    def __init__(
+        self,
+        supported_distros: SchedulerPluginSupportedDistros = None,
+        supported_regions: List[str] = None,
+        queue_constraints: SchedulerPluginQueueConstraints = None,
+        compute_resource_constraints: SchedulerPluginComputeResourceConstraints = None,
+        requires_sudo_privileges: bool = None,
+        supports_cluster_update: bool = None,
+        supported_parallel_cluster_versions: str = None,
+        **kwargs,
+    ):
+        super().__init__(**kwargs)
+        self.supported_distros = supported_distros
+        self.supported_regions = supported_regions
+        self.queue_constraints = queue_constraints
+        self.compute_resource_constraints = compute_resource_constraints
+        self.requires_sudo_privileges = Resource.init_param(requires_sudo_privileges, default=False)
+        self.supports_cluster_update = Resource.init_param(supports_cluster_update, default=True)
+        self.supported_parallel_cluster_versions = supported_parallel_cluster_versions
+
+    def _register_validators(self):
+        if self.supported_parallel_cluster_versions:
+            self._register_validator(
+                SupportedVersionsValidator,
+                installed_version=get_installed_version(),
+                supported_versions_string=self.supported_parallel_cluster_versions,
+            )
+
+
+class SchedulerPluginCloudFormationInfrastructure(Resource):
+    """Represent the CloudFormation infrastructure for a Scheduler Plugin."""
+
+    def __init__(self, template: str, **kwargs):
+        super().__init__(**kwargs)
+        self.template = replace_url_parameters(template)
+
+    def _register_validators(self):
+        self._register_validator(UrlValidator, url=self.template, fail_on_https_error=True)
+
+
+class SchedulerPluginClusterInfrastructure(Resource):
+    """Represent the ClusterInfastructure config for a Scheduler Plugin."""
+
+    def __init__(self, cloud_formation: SchedulerPluginCloudFormationInfrastructure = None, **kwargs):
+        super().__init__(**kwargs)
+        self.cloud_formation = cloud_formation
+
+
+class SchedulerPluginClusterSharedArtifact(Resource):
+    """Represent the ClusterSharedArtifact config for a Scheduler Plugin."""
+
+    def __init__(self, source: str, **kwargs):
+        super().__init__(**kwargs)
+        self.source = replace_url_parameters(source)
+
+    def _register_validators(self):
+        self._register_validator(UrlValidator, url=self.source)
+
+
+class SchedulerPluginPluginResources(Resource):
+    """Represent the PluginResources config for a Scheduler Plugin."""
+
+    def __init__(self, cluster_shared_artifacts: [SchedulerPluginClusterSharedArtifact], **kwargs):
+        super().__init__(**kwargs)
+        self.cluster_shared_artifacts = cluster_shared_artifacts
+
+
+class SchedulerPluginExecuteCommand(Resource):
+    """Represent the ExecuteCommand for a Scheduler Plugin."""
+
+    def __init__(self, command: str, **kwargs):
+        super().__init__(**kwargs)
+        self.command = command
+
+
+class SchedulerPluginEvent(Resource):
+    """Represent the Event config for a Scheduler Plugin."""
+
+    def __init__(self, execute_command: SchedulerPluginExecuteCommand, **kwargs):
+        super().__init__(**kwargs)
+        self.execute_command = execute_command
+
+
+class SchedulerPluginEvents(Resource):
+    """Represent the Events config for a Scheduler Plugin."""
+
+    def __init__(
+        self,
+        head_init: SchedulerPluginEvent = None,
+        head_configure: SchedulerPluginEvent = None,
+        head_finalize: SchedulerPluginEvent = None,
+        compute_init: SchedulerPluginEvent = None,
+        compute_configure: SchedulerPluginEvent = None,
+        compute_finalize: SchedulerPluginEvent = None,
+        head_cluster_update: SchedulerPluginEvent = None,
+        head_compute_fleet_start: SchedulerPluginEvent = None,
+        head_compute_fleet_stop: SchedulerPluginEvent = None,
+        **kwargs,
+    ):
+        super().__init__(**kwargs)
+        self.head_init = head_init
+        self.head_configure = head_configure
+        self.head_finalize = head_finalize
+        self.compute_init = compute_init
+        self.compute_configure = compute_configure
+        self.compute_finalize = compute_finalize
+        self.head_cluster_update = head_cluster_update
+        self.head_compute_fleet_start = head_compute_fleet_start
+        self.head_compute_fleet_stop = head_compute_fleet_stop
+
+
+class SchedulerPluginFile(Resource):
+    """Represent the Scheduler Plugin file resource."""
+
+    def __init__(
+        self, file_path: str, log_stream_name: str, node_type: str = None, timestamp_format: str = None, **kwargs
+    ):
+        super().__init__(**kwargs)
+        self.file_path = file_path
+        self.timestamp_format = Resource.init_param(timestamp_format, default="%Y-%m-%dT%H:%M:%S%z")
+        self.node_type = Resource.init_param(node_type, default="ALL")
+        self.log_stream_name = log_stream_name
+
+
+class SchedulerPluginLogs(Resource):
+    """Represent the Scheduler Plugin logs resource."""
+
+    def __init__(self, files: [SchedulerPluginFile], **kwargs):
+        super().__init__(**kwargs)
+        self.files = files
+
+
+class SchedulerPluginMonitoring(Resource):
+    """Represent the Scheduler Plugin monitoring resource."""
+
+    def __init__(self, logs: SchedulerPluginLogs, **kwargs):
+        super().__init__(**kwargs)
+        self.logs = logs
+
+
+class SchedulerPluginUser(Resource):
+    """Represent the Scheduler Plugin user resource."""
+
+    def __init__(self, name: str, enable_imds: bool = None, **kwargs):
+        super().__init__(**kwargs)
+        self.name = name
+        self.enable_imds = Resource.init_param(enable_imds, default=False)
+
+
+class SchedulerPluginDefinition(Resource):
+    """Represent the Scheduler Plugin scheduler definition."""
+
+    def __init__(
+        self,
+        plugin_interface_version: str,
+        events: SchedulerPluginEvents,
+        metadata: Dict = None,
+        requirements: SchedulerPluginRequirements = None,
+        cluster_infrastructure: SchedulerPluginClusterInfrastructure = None,
+        plugin_resources: SchedulerPluginPluginResources = None,
+        monitoring: SchedulerPluginMonitoring = None,
+        system_users: [SchedulerPluginUser] = None,
+        tags: List[Tag] = None,
+        **kwargs,
+    ):
+        super().__init__(**kwargs)
+        self.plugin_interface_version = plugin_interface_version
+        self.metadata = metadata
+        self.requirements = requirements
+        self.cluster_infrastructure = cluster_infrastructure
+        self.plugin_resources = plugin_resources
+        self.events = events
+        self.monitoring = monitoring
+        self.system_users = system_users
+        self.tags = tags
+
+
+class SchedulerPluginSettings(Resource):
+    """Represent the Scheduler Plugin settings."""
+
+    def __init__(
+        self,
+        scheduler_definition: SchedulerPluginDefinition,
+        grant_sudo_privileges: bool = None,
+        custom_settings: Dict = None,
+        **kwargs,
+    ):
+        super().__init__(**kwargs)
+        self.scheduler_definition = scheduler_definition
+        self.grant_sudo_privileges = Resource.init_param(grant_sudo_privileges, default=False)
+        self.custom_settings = custom_settings
+
+    def _register_validators(self):
+        self._register_validator(
+            SudoPrivilegesValidator,
+            grant_sudo_privileges=self.grant_sudo_privileges,
+            requires_sudo_privileges=self.scheduler_definition.requirements.requires_sudo_privileges
+            if self.scheduler_definition.requirements
+            else None,
+        )
+
+
+class SchedulerPluginScheduling(Resource):
+    """Represent a Scheduler Plugin Scheduling resource."""
+
+    def __init__(self, queues: List[SchedulerPluginQueue], settings: SchedulerPluginSettings, **kwargs):
+        super().__init__(**kwargs)
+        self.scheduler = "plugin"
         self.queues = queues
-        self.settings = settings or SlurmSettings(implied=True)
+        self.settings = settings
 
     def _register_validators(self):
         self._register_validator(
             DuplicateNameValidator, name_list=[queue.name for queue in self.queues], resource_name="Queue"
         )
+        self._register_validator(
+            MaxCountValidator,
+            resources_length=len(self.queues),
+            max_length=get_attr(
+                self.settings.scheduler_definition,
+                "requirements.queue_constraints.max_count",
+                default=MAX_NUMBER_OF_QUEUES,
+            ),
+            resource_name="SchedulerQueues",
+        )
+        for queue in self.queues:
+            self._register_validator(
+                MaxCountValidator,
+                resources_length=len(queue.compute_resources),
+                max_length=get_attr(
+                    self.settings.scheduler_definition,
+                    "requirements.compute_resource_constraints.max_count",
+                    default=MAX_NUMBER_OF_COMPUTE_RESOURCES,
+                ),
+                resource_name="ComputeResources",
+            )
 
 
-class SlurmClusterConfig(BaseClusterConfig):
-    """Represent the full Slurm Cluster configuration."""
+class SchedulerPluginClusterConfig(BaseClusterConfig):
+    """Represent the full Scheduler Plugin Cluster configuration."""
 
-    def __init__(self, cluster_name: str, scheduling: SlurmScheduling, **kwargs):
+    def __init__(self, cluster_name: str, scheduling: SchedulerPluginScheduling, **kwargs):
         super().__init__(cluster_name, **kwargs)
         self.scheduling = scheduling
         self.__image_dict = None
@@ -1507,25 +1919,125 @@ class SlurmClusterConfig(BaseClusterConfig):
                 result[instance_type_info.instance_type()] = instance_type_info.instance_type_data
         return result
 
+    def get_cluster_tags(self):
+        """Return tags configured in the root of the cluster config and under scheduler definition."""
+        return (self.tags if self.tags else []) + get_attr(
+            self.scheduling, "settings.scheduler_definition.tags", default=[]
+        )
+
     def _register_validators(self):
         super()._register_validators()
         self._register_validator(SchedulerOsValidator, scheduler=self.scheduling.scheduler, os=self.image.os)
         self._register_validator(
             HeadNodeImdsValidator, imds_secured=self.head_node.imds.secured, scheduler=self.scheduling.scheduler
         )
-        if self.scheduling.settings and self.scheduling.settings.dns and self.scheduling.settings.dns.hosted_zone_id:
-            self._register_validator(
-                HostedZoneValidator,
-                hosted_zone_id=self.scheduling.settings.dns.hosted_zone_id,
-                cluster_vpc=self.vpc_id,
-                cluster_name=self.cluster_name,
-            )
+        scheduler_definition = self.scheduling.settings.scheduler_definition
+        self._register_validator(
+            SchedulerPluginOsArchitectureValidator,
+            os=self.image.os,
+            architecture=self.head_node.architecture,
+            supported_x86=get_attr(scheduler_definition, "requirements.supported_distros.x86", default=SUPPORTED_OSES),
+            supported_arm64=get_attr(
+                scheduler_definition, "requirements.supported_distros.arm64", default=SUPPORTED_OSES
+            ),
+        )
+        self._register_validator(
+            SchedulerPluginRegionValidator,
+            region=self.region,
+            supported_regions=get_attr(scheduler_definition, "requirements.supported_regions"),
+        )
 
         checked_images = []
 
         for queue in self.scheduling.queues:
             self._register_validator(
-                ComputeResourceLaunchTemplateValidator, queue=queue, ami_id=self.image_dict[queue.name], tags=self.tags
+                ComputeResourceLaunchTemplateValidator,
+                queue=queue,
+                ami_id=self.image_dict[queue.name],
+                tags=self.get_cluster_tags(),
+            )
+            queue_image = self.image_dict[queue.name]
+            if queue_image not in checked_images and queue.queue_ami:
+                checked_images.append(queue_image)
+                self._register_validator(AmiOsCompatibleValidator, os=self.image.os, image_id=queue_image)
+            for compute_resource in queue.compute_resources:
+                if self.image_dict[queue.name]:
+                    self._register_validator(
+                        InstanceTypeBaseAMICompatibleValidator,
+                        instance_type=compute_resource.instance_type,
+                        image=queue_image,
+                    )
+                self._register_validator(
+                    InstanceArchitectureCompatibilityValidator,
+                    instance_type=compute_resource.instance_type,
+                    architecture=self.head_node.architecture,
+                )
+                self._register_validator(
+                    EfaOsArchitectureValidator,
+                    efa_enabled=compute_resource.efa.enabled,
+                    os=self.image.os,
+                    architecture=self.head_node.architecture,
+                )
+
+    @property
+    def image_dict(self):
+        """Return image dict of queues, key is queue name, value is image id."""
+        if self.__image_dict:
+            return self.__image_dict
+        self.__image_dict = {}
+
+        for queue in self.scheduling.queues:
+            self.__image_dict[queue.name] = queue.queue_ami or self.image.custom_ami or self.official_ami
+
+        return self.__image_dict
+
+
+class SlurmClusterConfig(BaseClusterConfig):
+    """Represent the full Slurm Cluster configuration."""
+
+    def __init__(self, cluster_name: str, scheduling: SlurmScheduling, **kwargs):
+        super().__init__(cluster_name, **kwargs)
+        self.scheduling = scheduling
+        self.__image_dict = None
+
+    def get_instance_types_data(self):
+        """Get instance type infos for all instance types used in the configuration file."""
+        result = {}
+        instance_type_info = self.head_node.instance_type_info
+        result[instance_type_info.instance_type()] = instance_type_info.instance_type_data
+        for queue in self.scheduling.queues:
+            for compute_resource in queue.compute_resources:
+                instance_type_info = compute_resource.instance_type_info
+                result[instance_type_info.instance_type()] = instance_type_info.instance_type_data
+        return result
+
+    def _register_validators(self):
+        super()._register_validators()
+        self._register_validator(SchedulerOsValidator, scheduler=self.scheduling.scheduler, os=self.image.os)
+        self._register_validator(
+            HeadNodeImdsValidator, imds_secured=self.head_node.imds.secured, scheduler=self.scheduling.scheduler
+        )
+        self._register_validator(
+            MixedSecurityGroupOverwriteValidator,
+            head_node_security_groups=self.head_node.networking.security_groups,
+            queues=self.scheduling.queues,
+        )
+        if self.scheduling.settings and self.scheduling.settings.dns and self.scheduling.settings.dns.hosted_zone_id:
+            self._register_validator(
+                HostedZoneValidator,
+                hosted_zone_id=self.scheduling.settings.dns.hosted_zone_id,
+                cluster_vpc=self.vpc_id,
+                cluster_name=self.cluster_name,
+            )
+
+        checked_images = []
+
+        for queue in self.scheduling.queues:
+            self._register_validator(
+                ComputeResourceLaunchTemplateValidator,
+                queue=queue,
+                ami_id=self.image_dict[queue.name],
+                tags=self.get_cluster_tags(),
             )
             queue_image = self.image_dict[queue.name]
             if queue_image not in checked_images and queue.queue_ami:
