@@ -12,6 +12,7 @@
 # This module contains all the classes representing the Resources objects.
 # These objects are obtained from the configuration file through a conversion based on the Schema classes.
 #
+import hashlib
 import json
 import logging
 import os
@@ -21,14 +22,17 @@ from copy import deepcopy
 from datetime import datetime
 from enum import Enum
 from typing import List, Optional, Set, Tuple
+from urllib.request import urlopen
 
 import pkg_resources
 import yaml
+from jinja2 import BaseLoader
+from jinja2.sandbox import SandboxedEnvironment
 from marshmallow import ValidationError
 
 from pcluster.aws.aws_api import AWSApi
 from pcluster.aws.common import AWSClientError, BadRequestError, LimitExceededError, StackNotFoundError, get_region
-from pcluster.config.cluster_config import BaseClusterConfig, SlurmScheduling, Tag
+from pcluster.config.cluster_config import BaseClusterConfig, SchedulerPluginScheduling, SlurmScheduling, Tag
 from pcluster.config.common import ValidatorSuppressor
 from pcluster.config.config_patch import ConfigPatch
 from pcluster.constants import (
@@ -59,10 +63,10 @@ from pcluster.models.common import (
     upload_archive,
 )
 from pcluster.models.compute_fleet_status_manager import ComputeFleetStatus, ComputeFleetStatusManager
-from pcluster.models.s3_bucket import S3Bucket, S3BucketFactory, S3FileFormat, create_s3_presigned_url
+from pcluster.models.s3_bucket import S3Bucket, S3BucketFactory, S3FileFormat, create_s3_presigned_url, parse_bucket_url
 from pcluster.schemas.cluster_schema import ClusterSchema
 from pcluster.templates.cdk_builder import CDKTemplateBuilder
-from pcluster.utils import datetime_to_epoch, generate_random_name_with_prefix, get_installed_version, grouper
+from pcluster.utils import datetime_to_epoch, generate_random_name_with_prefix, get_attr, get_installed_version, grouper
 from pcluster.validators.common import FailureLevel, ValidationResult
 
 # pylint: disable=C0302
@@ -271,14 +275,14 @@ class Cluster:
     def compute_fleet_status_with_last_updated_time(self):
         """Status of the cluster compute fleet and the last compute fleet status updated time."""
         if self.stack.is_working_status or self.stack.status == "UPDATE_IN_PROGRESS":
-            if self.stack.scheduler == "slurm":
-                compute_fleet_status_manager = ComputeFleetStatusManager(self.name)
-                status, last_updated_time = compute_fleet_status_manager.get_status_with_last_updated_time()
-            else:  # scheduler is AWS Batch:
+            if self.stack.scheduler == "awsbatch":
                 status = ComputeFleetStatus(
                     AWSApi.instance().batch.get_compute_environment_state(self.stack.batch_compute_environment)
                 )
                 last_updated_time = None
+            else:
+                compute_fleet_status_manager = ComputeFleetStatusManager(self.name)
+                status, last_updated_time = compute_fleet_status_manager.get_status_with_last_updated_time()
             return status, last_updated_time
         else:
             LOGGER.info(
@@ -377,7 +381,7 @@ class Cluster:
                 self.bucket.delete_s3_artifacts()
             raise _cluster_error_mapper(e, str(e))
 
-    def _load_config(self, cluster_config: dict) -> ClusterSchema:
+    def _load_config(self, cluster_config: dict) -> BaseClusterConfig:
         """Load the config and catch / translate any errors that occur during loading."""
         try:
             return ClusterSchema(cluster_name=self.name).load(cluster_config)
@@ -387,12 +391,14 @@ class Cluster:
             validation_failures = [ValidationResult(data, FailureLevel.ERROR, validator_type="ConfigSchemaValidator")]
             raise ConfigValidationError("Invalid cluster configuration.", validation_failures=validation_failures)
 
-    def validate_create_request(self, validator_suppressors, validation_failure_level):
+    def validate_create_request(self, validator_suppressors, validation_failure_level, dry_run=False):
         """Validate a create cluster request."""
         self._validate_no_existing_stack()
         self.config, ignored_validation_failures = self._validate_and_parse_config(
             validator_suppressors, validation_failure_level
         )
+        if dry_run and isinstance(self.config.scheduling, SchedulerPluginScheduling):
+            self._render_and_upload_scheduler_plugin_template(dry_run=dry_run)
         return ignored_validation_failures
 
     def _validate_no_existing_stack(self):
@@ -483,18 +489,70 @@ class Cluster:
             if self.template_body:
                 self.bucket.upload_cfn_template(self.template_body, PCLUSTER_S3_ARTIFACTS_DICT.get("template_name"))
 
-            if isinstance(self.config.scheduling, SlurmScheduling):
+            if isinstance(self.config.scheduling, (SlurmScheduling, SchedulerPluginScheduling)):
                 # upload instance types data
                 self.bucket.upload_config(
                     self.config.get_instance_types_data(),
                     PCLUSTER_S3_ARTIFACTS_DICT.get("instance_types_data_name"),
                     format=S3FileFormat.JSON,
                 )
+
+            if isinstance(self.config.scheduling, SchedulerPluginScheduling):
+                self._render_and_upload_scheduler_plugin_template()
+        except BadRequestClusterActionError:
+            raise
         except Exception as e:
             message = f"Unable to upload cluster resources to the S3 bucket {self.bucket.name} due to exception: {e}"
             LOGGER.error(message)
             raise _cluster_error_mapper(e, message)
 
+    def _render_and_upload_scheduler_plugin_template(self, dry_run=False):
+        scheduler_plugin_template = get_attr(
+            self.config, "scheduling.settings.scheduler_definition.cluster_infrastructure.cloud_formation.template"
+        )
+        if not scheduler_plugin_template:
+            return
+
+        try:
+            LOGGER.info("Downloading scheduler plugin CloudFormation template from %s", scheduler_plugin_template)
+            if scheduler_plugin_template.startswith("s3"):
+                bucket_parsing_result = parse_bucket_url(scheduler_plugin_template)
+                result = AWSApi.instance().s3.get_object(
+                    bucket_name=bucket_parsing_result["bucket_name"], key=bucket_parsing_result["object_key"]
+                )
+                file_content = result["Body"].read().decode("utf-8")
+            else:
+                with urlopen(  # nosec nosemgrep - scheduler_plugin_template url is properly validated
+                    scheduler_plugin_template
+                ) as f:
+                    file_content = f.read().decode("utf-8")
+        except Exception as e:
+            raise BadRequestClusterActionError(
+                f"Error while downloading scheduler plugin artifacts from '{scheduler_plugin_template}': {str(e)}"
+            ) from e
+
+        # jinja rendering
+        try:
+            LOGGER.info("Rendering the following scheduler plugin CloudFormation template:\n%s", file_content)
+            environment = SandboxedEnvironment(loader=BaseLoader)
+            environment.filters["hash"] = (
+                lambda value: hashlib.sha1(value.encode()).hexdigest()[0:16].capitalize()  # nosec nosemgrep
+            )
+            template = environment.from_string(file_content)
+            rendered_template = template.render(
+                cluster_configuration=ClusterSchema(cluster_name=self.name).dump(deepcopy(self.config)),
+                cluster_name=self.name,
+                instance_types_info=self.config.get_instance_types_data(),
+            )
+        except Exception as e:
+            raise BadRequestClusterActionError(
+                f"Error while rendering scheduler plugin template '{scheduler_plugin_template}': {str(e)}"
+            ) from e
+        if not dry_run:
+            self.bucket.upload_cfn_template(
+                rendered_template, PCLUSTER_S3_ARTIFACTS_DICT["scheduler_plugin_template_name"], S3FileFormat.TEXT
+            )
+
     def delete(self, keep_logs: bool = True):
         """Delete cluster preserving log groups."""
         try:
@@ -726,6 +784,7 @@ class Cluster:
         validator_suppressors: Set[ValidatorSuppressor] = None,
         validation_failure_level: FailureLevel = FailureLevel.ERROR,
         force: bool = False,
+        dry_run: bool = False,
     ):
         """Validate a cluster update request."""
         self._validate_cluster_exists()
@@ -735,6 +794,11 @@ class Cluster:
         )
         changes = self._validate_patch(force, target_config)
 
+        self._validate_scheduling_update(changes, target_config)
+
+        if dry_run and isinstance(self.config.scheduling, SchedulerPluginScheduling):
+            self._render_and_upload_scheduler_plugin_template(dry_run=dry_run)
+
         return target_config, changes, ignored_validation_failures
 
     def _validate_patch(self, force, target_config):
@@ -827,7 +891,13 @@ class Cluster:
 
     def _get_cfn_tags(self):
         """Return tag list in the format expected by CFN."""
-        return [{"Key": tag.key, "Value": tag.value} for tag in self.config.tags]
+        cluster_tags = [{"Key": tag.key, "Value": tag.value} for tag in self.config.tags]
+        if self.config.scheduling.scheduler == "plugin":
+            scheduler_plugin_tags = get_attr(self.config, "scheduling.settings.scheduler_definition.tags")
+            if scheduler_plugin_tags:
+                custom_scheduler_plugin_tags = [{"Key": tag.key, "Value": tag.value} for tag in scheduler_plugin_tags]
+                cluster_tags += custom_scheduler_plugin_tags
+        return cluster_tags
 
     def export_logs(
         self,
@@ -1025,3 +1095,59 @@ class Cluster:
     def _stack_events_stream_name(self):
         """Return the name of the stack events log stream."""
         return STACK_EVENTS_LOG_STREAM_NAME_FORMAT.format(self.stack_name)
+
+    def _validate_scheduling_update(self, changes, target_config):
+        """Update of Scheduling is not supported when SupportsClusterUpdate of the scheduler plugin is set to false."""
+        # target_config.source_config.get("Scheduling") != self.config.source_config.get("Scheduling") doesn't mean
+        # there's changes in the config, if queue list in the scheduling dict has different order, target_config dict
+        # and original config dict may be different.
+        if (
+            self.config.scheduling.scheduler == "plugin"
+            and get_attr(self.config, "scheduling.settings.scheduler_definition.requirements.supports_cluster_update")
+            is False
+            and target_config.source_config.get("Scheduling") != self.config.source_config.get("Scheduling")
+        ):
+            scheduling_changes = []
+            # Example format of changes:
+            # changes = [
+            #     ["param_path", "parameter", "old value", "new value", "check", "reason", "action_needed"],
+            #     [
+            #         ["HeadNode", "Iam"],
+            #         "AdditionalIamPolicies",
+            #         None,
+            #         {"Policy": "arn:aws:iam::aws:policy/FakePolicy"},
+            #         "SUCCEEDED",
+            #         "-",
+            #         None,
+            #     ],
+            #     [
+            #         ["HeadNode", "Iam"],
+            #         "AdditionalIamPolicies",
+            #         {"Policy": "arn:aws:iam::aws:policy/AmazonS3ReadOnlyAccess"},
+            #         None,
+            #         "SUCCEEDED",
+            #         "-",
+            #         None,
+            #     ],
+            #     [
+            #         ["Scheduling", "SchedulerQueues[queue1]", "ComputeResources[compute-resource1]"],
+            #         "InstanceType",
+            #         "c5.2xlarge",
+            #         "c5.xlarge",
+            #         "SUCCEEDED",
+            #         "-",
+            #         None,
+            #     ],
+            # ]
+
+            # The first element of changes is:
+            # ["param_path", "parameter", "old value", "new value", "check", "reason", "action_needed"]
+            for change in changes[1:]:
+                if change[0][0] == "Scheduling":  # check if the param_path of the change start from Scheduling
+                    scheduling_changes.append(change)
+            if len(scheduling_changes) >= 1:
+                raise ClusterUpdateError(
+                    "Update failure: The scheduler plugin used for this cluster does not support updating the "
+                    "scheduling configuration.",
+                    [changes[0]] + scheduling_changes,
+                )
