@@ -39,6 +39,11 @@ fi
 Content-Type: text/cloud-config; charset=us-ascii
 MIME-Version: 1.0
 
+package_update: false
+package_upgrade: false
+repo_upgrade: none
+
+datasource_list: [ Ec2, None ]
 output:
   all: "| tee -a /var/log/cloud-init-output.log | logger -t user-data -s 2>/dev/console"
 write_files:
@@ -77,10 +82,16 @@ write_files:
           "enable_intel_hpc_platform": "${IntelHPCPlatform}",
           "cw_logging_enabled": "${CWLoggingEnabled}",
           "scheduler_queue_name": "${QueueName}",
+          "scheduler_compute_resource_name": "${ComputeResourceName}",
+          "enable_efa_gdr": "${EnableEfaGdr}",
           "custom_node_package": "${CustomNodePackage}",
-          "custom_awsbatchcli_package": "${CustomAwsBatchCliPackage}"
-        },
-        "run_list": "recipe[aws-parallelcluster::${Scheduler}_config]"
+          "custom_awsbatchcli_package": "${CustomAwsBatchCliPackage}",
+          "use_private_hostname": "${UsePrivateHostname}",
+          "head_node_private_ip": "${HeadNodePrivateIp}",
+          "directory_service": {
+            "enabled": "${DirectoryServiceEnabled}"
+          }
+        }
       }
   - path: /etc/chef/client.rb
     permissions: '0644'
@@ -100,14 +111,10 @@ MIME-Version: 1.0
 
 function error_exit
 {
-  instance_id=$(curl --retry 3 --retry-delay 0 --silent --fail http://169.254.169.254/latest/meta-data/instance-id)
-  log_dir=/home/logs/compute
-  mkdir -p ${!log_dir}
-  echo "Reporting instance as unhealthy and dumping logs to ${!log_dir}/${!instance_id}.tar.gz"
-  tar -czf ${!log_dir}/${!instance_id}.tar.gz /var/log
-  # TODO: add possibility to disable this behavior
+  echo "Bootstrap failed with error: $1"
   # wait logs flush before signaling the failure
   sleep 10
+  # TODO: add possibility to override this behavior and keep the instance for debugging
   shutdown -h now
   exit 1
 }
@@ -160,14 +167,14 @@ cd /tmp
 
 mkdir -p /etc/chef/ohai/hints
 touch /etc/chef/ohai/hints/ec2.json
-jq --argfile f1 /tmp/dna.json --argfile f2 /tmp/extra.json -n '$f1 + $f2 | .cluster = $f1.cluster + $f2.cluster' > /etc/chef/dna.json || ( echo "jq not installed or invalid extra_json"; cp /tmp/dna.json /etc/chef/dna.json)
+jq --argfile f1 /tmp/dna.json --argfile f2 /tmp/extra.json -n '$f1 * $f2' > /etc/chef/dna.json || ( echo "jq not installed or invalid extra_json"; cp /tmp/dna.json /etc/chef/dna.json)
 {
   pushd /etc/chef &&
-  chef-client --local-mode --config /etc/chef/client.rb --log_level auto --force-formatter --no-color --chef-zero-port 8889 --json-attributes /etc/chef/dna.json --override-runlist aws-parallelcluster::prep_env &&
+  cinc-client --local-mode --config /etc/chef/client.rb --log_level info --force-formatter --no-color --chef-zero-port 8889 --json-attributes /etc/chef/dna.json --override-runlist aws-parallelcluster::init &&
   /opt/parallelcluster/scripts/fetch_and_run -preinstall &&
-  chef-client --local-mode --config /etc/chef/client.rb --log_level auto --force-formatter --no-color --chef-zero-port 8889 --json-attributes /etc/chef/dna.json &&
+  cinc-client --local-mode --config /etc/chef/client.rb --log_level info --force-formatter --no-color --chef-zero-port 8889 --json-attributes /etc/chef/dna.json --override-runlist aws-parallelcluster::config &&
   /opt/parallelcluster/scripts/fetch_and_run -postinstall &&
-  chef-client --local-mode --config /etc/chef/client.rb --log_level auto --force-formatter --no-color --chef-zero-port 8889 --json-attributes /etc/chef/dna.json --override-runlist aws-parallelcluster::finalize &&
+  cinc-client --local-mode --config /etc/chef/client.rb --log_level info --force-formatter --no-color --chef-zero-port 8889 --json-attributes /etc/chef/dna.json --override-runlist aws-parallelcluster::finalize &&
   popd
 } || error_exit 'Failed to run bootstrap recipes. If --norollback was specified, check /var/log/cfn-init.log and /var/log/cloud-init-output.log.'
 
