@@ -15,10 +15,15 @@
 #
 
 import copy
+import logging
 import re
+from urllib.request import urlopen
 
-from marshmallow import ValidationError, fields, post_load, pre_dump, validate, validates, validates_schema
+import yaml
+from marshmallow import ValidationError, fields, post_load, pre_dump, pre_load, validate, validates, validates_schema
+from yaml import YAMLError
 
+from pcluster.aws.aws_api import AWSApi
 from pcluster.config.cluster_config import (
     AdditionalPackages,
     AmiSearchFilters,
@@ -28,7 +33,6 @@ from pcluster.config.cluster_config import (
     AwsBatchQueueNetworking,
     AwsBatchScheduling,
     AwsBatchSettings,
-    BaseClusterConfig,
     CapacityType,
     CloudWatchDashboards,
     CloudWatchLogs,
@@ -39,6 +43,7 @@ from pcluster.config.cluster_config import (
     CustomActions,
     Dashboards,
     Dcv,
+    DirectoryService,
     Dns,
     Efa,
     EphemeralVolume,
@@ -59,6 +64,28 @@ from pcluster.config.cluster_config import (
     Roles,
     RootVolume,
     S3Access,
+    SchedulerPluginCloudFormationInfrastructure,
+    SchedulerPluginClusterConfig,
+    SchedulerPluginClusterInfrastructure,
+    SchedulerPluginClusterSharedArtifact,
+    SchedulerPluginComputeResource,
+    SchedulerPluginComputeResourceConstraints,
+    SchedulerPluginDefinition,
+    SchedulerPluginEvent,
+    SchedulerPluginEvents,
+    SchedulerPluginExecuteCommand,
+    SchedulerPluginFile,
+    SchedulerPluginLogs,
+    SchedulerPluginMonitoring,
+    SchedulerPluginPluginResources,
+    SchedulerPluginQueue,
+    SchedulerPluginQueueConstraints,
+    SchedulerPluginQueueNetworking,
+    SchedulerPluginRequirements,
+    SchedulerPluginScheduling,
+    SchedulerPluginSettings,
+    SchedulerPluginSupportedDistros,
+    SchedulerPluginUser,
     SharedEbs,
     SharedEfs,
     SharedFsx,
@@ -77,10 +104,10 @@ from pcluster.constants import (
     EBS_VOLUME_SIZE_DEFAULT,
     FSX_HDD_THROUGHPUT,
     FSX_SSD_THROUGHPUT,
-    MAX_NUMBER_OF_COMPUTE_RESOURCES,
-    MAX_NUMBER_OF_QUEUES,
+    SCHEDULER_PLUGIN_MAX_NUMBER_OF_USERS,
     SUPPORTED_OSES,
 )
+from pcluster.models.s3_bucket import parse_bucket_url
 from pcluster.schemas.common_schema import (
     AdditionalIamPolicySchema,
     BaseDevSettingsSchema,
@@ -93,6 +120,10 @@ from pcluster.validators.cluster_validators import FSX_MESSAGES
 
 # pylint: disable=C0302
 
+
+LOGGER = logging.getLogger(__name__)
+
+
 # ---------------------- Storage ---------------------- #
 
 
@@ -117,7 +148,6 @@ class HeadNodeRootVolumeSchema(BaseSchema):
             )
         }
     )
-    kms_key_id = fields.Str(metadata={"update_policy": UpdatePolicy.UNSUPPORTED})
     throughput = fields.Int(metadata={"update_policy": UpdatePolicy.SUPPORTED})
     encrypted = fields.Bool(metadata={"update_policy": UpdatePolicy.UNSUPPORTED})
     delete_on_termination = fields.Bool(metadata={"update_policy": UpdatePolicy.SUPPORTED})
@@ -345,7 +375,8 @@ class FsxLustreSettingsSchema(BaseSchema):
         validate=validate.Regexp(r"^fs-[0-9a-z]{17}$"), metadata={"update_policy": UpdatePolicy.UNSUPPORTED}
     )
     auto_import_policy = fields.Str(
-        validate=validate.OneOf(["NEW", "NEW_CHANGED"]), metadata={"update_policy": UpdatePolicy.UNSUPPORTED}
+        validate=validate.OneOf(["NEW", "NEW_CHANGED", "NEW_CHANGED_DELETED"]),
+        metadata={"update_policy": UpdatePolicy.UNSUPPORTED},
     )
     drive_cache_type = fields.Str(
         validate=validate.OneOf(["READ"]), metadata={"update_policy": UpdatePolicy.UNSUPPORTED}
@@ -583,6 +614,15 @@ class AwsBatchQueueNetworkingSchema(QueueNetworkingSchema):
         return AwsBatchQueueNetworking(**data)
 
 
+class SchedulerPluginQueueNetworkingSchema(SlurmQueueNetworkingSchema):
+    """Represent the schema of the Networking, child of Scheduler Plugin Queue."""
+
+    @post_load
+    def make_resource(self, data, **kwargs):
+        """Generate resource."""
+        return SchedulerPluginQueueNetworking(**data)
+
+
 class SshSchema(BaseSchema):
     """Represent the schema of the SSH."""
 
@@ -612,7 +652,7 @@ class EfaSchema(BaseSchema):
     """Represent the schema of EFA for a Compute Resource."""
 
     enabled = fields.Bool(metadata={"update_policy": UpdatePolicy.COMPUTE_FLEET_STOP})
-    gdr_support = fields.Bool(metadata={"update_policy": UpdatePolicy.IGNORED})
+    gdr_support = fields.Bool(metadata={"update_policy": UpdatePolicy.COMPUTE_FLEET_STOP})
 
     @post_load
     def make_resource(self, data, **kwargs):
@@ -1012,6 +1052,17 @@ class AwsBatchComputeResourceSchema(_ComputeResourceSchema):
         return AwsBatchComputeResource(**data)
 
 
+class SchedulerPluginComputeResourceSchema(SlurmComputeResourceSchema):
+    """Represent the schema of the Scheduler Plugin ComputeResource."""
+
+    custom_settings = fields.Dict(metadata={"update_policy": UpdatePolicy.COMPUTE_FLEET_STOP})
+
+    @post_load
+    def make_resource(self, data, **kwargs):
+        """Generate resource."""
+        return SchedulerPluginComputeResource(**data)
+
+
 class ComputeSettingsSchema(BaseSchema):
     """Represent the schema of the compute_settings schedulers queues."""
 
@@ -1033,21 +1084,25 @@ class BaseQueueSchema(BaseSchema):
     )
 
 
-class SlurmQueueSchema(BaseQueueSchema):
-    """Represent the schema of a Slurm Queue."""
+class _CommonQueueSchema(BaseQueueSchema):
+    """Represent the schema of common part between Slurm and Scheduler Plugin Queue."""
 
     compute_settings = fields.Nested(ComputeSettingsSchema, metadata={"update_policy": UpdatePolicy.COMPUTE_FLEET_STOP})
-    compute_resources = fields.Nested(
-        SlurmComputeResourceSchema,
-        many=True,
-        validate=validate.Length(max=MAX_NUMBER_OF_COMPUTE_RESOURCES),
-        metadata={"update_policy": UpdatePolicy.COMPUTE_FLEET_STOP, "update_key": "Name"},
-    )
     custom_actions = fields.Nested(
         QueueCustomActionsSchema, metadata={"update_policy": UpdatePolicy.COMPUTE_FLEET_STOP}
     )
     iam = fields.Nested(QueueIamSchema, metadata={"update_policy": UpdatePolicy.SUPPORTED})
     image = fields.Nested(QueueImageSchema, metadata={"update_policy": UpdatePolicy.COMPUTE_FLEET_STOP})
+
+
+class SlurmQueueSchema(_CommonQueueSchema):
+    """Represent the schema of a Slurm Queue."""
+
+    compute_resources = fields.Nested(
+        SlurmComputeResourceSchema,
+        many=True,
+        metadata={"update_policy": UpdatePolicy.COMPUTE_FLEET_STOP, "update_key": "Name"},
+    )
     networking = fields.Nested(
         SlurmQueueNetworkingSchema, required=True, metadata={"update_policy": UpdatePolicy.COMPUTE_FLEET_STOP}
     )
@@ -1077,11 +1132,31 @@ class AwsBatchQueueSchema(BaseQueueSchema):
         return AwsBatchQueue(**data)
 
 
+class SchedulerPluginQueueSchema(_CommonQueueSchema):
+    """Represent the schema of a Scheduler Plugin Queue."""
+
+    compute_resources = fields.Nested(
+        SchedulerPluginComputeResourceSchema,
+        many=True,
+        metadata={"update_policy": UpdatePolicy.COMPUTE_FLEET_STOP, "update_key": "Name"},
+    )
+    networking = fields.Nested(
+        SchedulerPluginQueueNetworkingSchema, required=True, metadata={"update_policy": UpdatePolicy.COMPUTE_FLEET_STOP}
+    )
+    custom_settings = fields.Dict(metadata={"update_policy": UpdatePolicy.COMPUTE_FLEET_STOP})
+
+    @post_load
+    def make_resource(self, data, **kwargs):
+        """Generate resource."""
+        return SchedulerPluginQueue(**data)
+
+
 class DnsSchema(BaseSchema):
     """Represent the schema of Dns Settings."""
 
-    disable_managed_dns = fields.Bool(metadata={"update_policy": UpdatePolicy.COMPUTE_FLEET_STOP})
+    disable_managed_dns = fields.Bool(metadata={"update_policy": UpdatePolicy.UNSUPPORTED})
     hosted_zone_id = fields.Str(metadata={"update_policy": UpdatePolicy.UNSUPPORTED})
+    use_ec2_hostnames = fields.Bool(metadata={"update_policy": UpdatePolicy.UNSUPPORTED})
 
     @post_load
     def make_resource(self, data, **kwargs):
@@ -1110,12 +1185,320 @@ class AwsBatchSettingsSchema(BaseSchema):
         return AwsBatchSettings(**data)
 
 
+class SchedulerPluginSupportedDistrosSchema(BaseSchema):
+    """Represent the schema for SupportedDistros in a Scheduler Plugin."""
+
+    x86 = fields.List(fields.Str(), metadata={"update_policy": UpdatePolicy.UNSUPPORTED})
+    arm64 = fields.List(fields.Str(), metadata={"update_policy": UpdatePolicy.UNSUPPORTED})
+
+    @post_load
+    def make_resource(self, data, **kwargs):
+        """Generate resource."""
+        return SchedulerPluginSupportedDistros(**data)
+
+
+class SchedulerPluginQueueConstraintsSchema(BaseSchema):
+    """Represent the schema for QueueConstraints in a Scheduler Plugin."""
+
+    max_count = fields.Int(metadata={"update_policy": UpdatePolicy.UNSUPPORTED})
+
+    @post_load
+    def make_resource(self, data, **kwargs):
+        """Generate resource."""
+        return SchedulerPluginQueueConstraints(**data)
+
+
+class SchedulerPluginComputeResourceConstraintsSchema(BaseSchema):
+    """Represent the schema for ComputeResourceConstraints in a Scheduler Plugin."""
+
+    max_count = fields.Int(metadata={"update_policy": UpdatePolicy.UNSUPPORTED})
+
+    @post_load
+    def make_resource(self, data, **kwargs):
+        """Generate resource."""
+        return SchedulerPluginComputeResourceConstraints(**data)
+
+
+class SchedulerPluginRequirementsSchema(BaseSchema):
+    """Represent the schema for Requirements in a Scheduler Plugin."""
+
+    supported_distros = fields.Nested(
+        SchedulerPluginSupportedDistrosSchema, metadata={"update_policy": UpdatePolicy.UNSUPPORTED}
+    )
+    supported_regions = fields.List(fields.Str(), metadata={"update_policy": UpdatePolicy.UNSUPPORTED})
+    queue_constraints = fields.Nested(
+        SchedulerPluginQueueConstraintsSchema, metadata={"update_policy": UpdatePolicy.UNSUPPORTED}
+    )
+    compute_resource_constraints = fields.Nested(
+        SchedulerPluginComputeResourceConstraintsSchema, metadata={"update_policy": UpdatePolicy.UNSUPPORTED}
+    )
+    requires_sudo_privileges = fields.Bool(metadata={"update_policy": UpdatePolicy.UNSUPPORTED})
+    supports_cluster_update = fields.Bool(metadata={"update_policy": UpdatePolicy.UNSUPPORTED})
+    supported_parallel_cluster_versions = fields.Str(
+        metadata={"update_policy": UpdatePolicy.UNSUPPORTED},
+        validate=validate.Regexp(r"^((>|<|>=|<=)?[0-9]+\.[0-9]+\.[0-9]+,\s*)*(>|<|>=|<=)?[0-9]+\.[0-9]+\.[0-9]+$"),
+    )
+
+    @post_load
+    def make_resource(self, data, **kwargs):
+        """Generate resource."""
+        return SchedulerPluginRequirements(**data)
+
+
+class SchedulerPluginCloudFormationClusterInfrastructureSchema(BaseSchema):
+    """Represent the CloudFormation section of the Scheduler Plugin ClusterInfrastructure schema."""
+
+    template = fields.Str(required=True, metadata={"update_policy": UpdatePolicy.UNSUPPORTED})
+
+    @post_load
+    def make_resource(self, data, **kwargs):
+        """Generate resource."""
+        return SchedulerPluginCloudFormationInfrastructure(**data)
+
+
+class SchedulerPluginClusterInfrastructureSchema(BaseSchema):
+    """Represent the schema for ClusterInfrastructure schema in a Scheduler Plugin."""
+
+    cloud_formation = fields.Nested(
+        SchedulerPluginCloudFormationClusterInfrastructureSchema,
+        required=True,
+        metadata={"update_policy": UpdatePolicy.UNSUPPORTED},
+    )
+
+    @post_load
+    def make_resource(self, data, **kwargs):
+        """Generate resource."""
+        return SchedulerPluginClusterInfrastructure(**data)
+
+
+class SchedulerPluginClusterSharedArtifactSchema(BaseSchema):
+    """Represent the schema for Cluster Shared Artifact in a Scheduler Plugin."""
+
+    source = fields.Str(required=True, metadata={"update_policy": UpdatePolicy.UNSUPPORTED})
+
+    @post_load
+    def make_resource(self, data, **kwargs):
+        """Generate resource."""
+        return SchedulerPluginClusterSharedArtifact(**data)
+
+
+class SchedulerPluginResourcesSchema(BaseSchema):
+    """Represent the schema for Plugin Resouces in a Scheduler Plugin."""
+
+    cluster_shared_artifacts = fields.Nested(
+        SchedulerPluginClusterSharedArtifactSchema,
+        many=True,
+        required=True,
+        metadata={"update_policy": UpdatePolicy.UNSUPPORTED, "update_key": "Source"},
+    )
+
+    @post_load
+    def make_resource(self, data, **kwargs):
+        """Generate resource."""
+        return SchedulerPluginPluginResources(**data)
+
+
+class SchedulerPluginExecuteCommandSchema(BaseSchema):
+    """Represent the schema for ExecuteCommand in a Scheduler Plugin."""
+
+    command = fields.Str(required=True, metadata={"update_policy": UpdatePolicy.UNSUPPORTED})
+
+    @post_load
+    def make_resource(self, data, **kwargs):
+        """Generate resource."""
+        return SchedulerPluginExecuteCommand(**data)
+
+
+class SchedulerPluginEventSchema(BaseSchema):
+    """Represent the schema for Event in a Scheduler Plugin."""
+
+    execute_command = fields.Nested(
+        SchedulerPluginExecuteCommandSchema, required=True, metadata={"update_policy": UpdatePolicy.UNSUPPORTED}
+    )
+
+    @post_load
+    def make_resource(self, data, **kwargs):
+        """Generate resource."""
+        return SchedulerPluginEvent(**data)
+
+
+class SchedulerPluginEventsSchema(BaseSchema):
+    """Represent the schema for Events in a Scheduler Plugin."""
+
+    head_init = fields.Nested(SchedulerPluginEventSchema, metadata={"update_policy": UpdatePolicy.UNSUPPORTED})
+    head_configure = fields.Nested(SchedulerPluginEventSchema, metadata={"update_policy": UpdatePolicy.UNSUPPORTED})
+    head_finalize = fields.Nested(SchedulerPluginEventSchema, metadata={"update_policy": UpdatePolicy.UNSUPPORTED})
+    compute_init = fields.Nested(SchedulerPluginEventSchema, metadata={"update_policy": UpdatePolicy.UNSUPPORTED})
+    compute_configure = fields.Nested(SchedulerPluginEventSchema, metadata={"update_policy": UpdatePolicy.UNSUPPORTED})
+    compute_finalize = fields.Nested(SchedulerPluginEventSchema, metadata={"update_policy": UpdatePolicy.UNSUPPORTED})
+    head_cluster_update = fields.Nested(
+        SchedulerPluginEventSchema, metadata={"update_policy": UpdatePolicy.UNSUPPORTED}
+    )
+    head_compute_fleet_start = fields.Nested(
+        SchedulerPluginEventSchema, metadata={"update_policy": UpdatePolicy.UNSUPPORTED}
+    )
+    head_compute_fleet_stop = fields.Nested(
+        SchedulerPluginEventSchema, metadata={"update_policy": UpdatePolicy.UNSUPPORTED}
+    )
+
+    @post_load
+    def make_resource(self, data, **kwargs):
+        """Generate resource."""
+        return SchedulerPluginEvents(**data)
+
+
+class SchedulerPluginFileSchema(BaseSchema):
+    """Represent the schema of the Scheduler Plugin."""
+
+    file_path = fields.Str(
+        required=True, validate=get_field_validator("file_path"), metadata={"update_policy": UpdatePolicy.UNSUPPORTED}
+    )
+    timestamp_format = fields.Str(metadata={"update_policy": UpdatePolicy.UNSUPPORTED})
+    node_type = fields.Str(
+        metadata={"update_policy": UpdatePolicy.UNSUPPORTED}, validate=validate.OneOf(["HEAD", "COMPUTE", "ALL"])
+    )
+    log_stream_name = fields.Str(
+        required=True, metadata={"update_policy": UpdatePolicy.UNSUPPORTED}, validate=validate.Regexp(r"^[^:*]*$")
+    )
+
+    @post_load
+    def make_resource(self, data, **kwargs):
+        """Generate resource."""
+        return SchedulerPluginFile(**data)
+
+
+class SchedulerPluginLogsSchema(BaseSchema):
+    """Represent the schema of the Scheduler Plugin Logs."""
+
+    files = fields.Nested(
+        SchedulerPluginFileSchema,
+        required=True,
+        many=True,
+        metadata={"update_policy": UpdatePolicy.UNSUPPORTED, "update_key": "FilePath"},
+    )
+
+    @post_load
+    def make_resource(self, data, **kwargs):
+        """Generate resource."""
+        return SchedulerPluginLogs(**data)
+
+
+class SchedulerPluginMonitoringSchema(BaseSchema):
+    """Represent the schema of the Scheduler plugin Monitoring."""
+
+    logs = fields.Nested(SchedulerPluginLogsSchema, required=True, metadata={"update_policy": UpdatePolicy.UNSUPPORTED})
+
+    @post_load
+    def make_resource(self, data, **kwargs):
+        """Generate resource."""
+        return SchedulerPluginMonitoring(**data)
+
+
+class SchedulerPluginUserSchema(BaseSchema):
+    """Represent the schema of the Scheduler Plugin."""
+
+    name = fields.Str(required=True, metadata={"update_policy": UpdatePolicy.UNSUPPORTED})
+    enable_imds = fields.Bool(metadata={"update_policy": UpdatePolicy.UNSUPPORTED})
+
+    @post_load
+    def make_resource(self, data, **kwargs):
+        """Generate resource."""
+        return SchedulerPluginUser(**data)
+
+
+class SchedulerPluginDefinitionSchema(BaseSchema):
+    """Represent the schema of the Scheduler Plugin SchedulerDefinition."""
+
+    plugin_interface_version = fields.Str(
+        required=True, metadata={"update_policy": UpdatePolicy.UNSUPPORTED}, validate=validate.OneOf(["1.0"])
+    )
+    metadata = fields.Dict(metadata={"update_policy": UpdatePolicy.UNSUPPORTED})
+    requirements = fields.Nested(
+        SchedulerPluginRequirementsSchema, metadata={"update_policy": UpdatePolicy.UNSUPPORTED}
+    )
+    cluster_infrastructure = fields.Nested(
+        SchedulerPluginClusterInfrastructureSchema, metadata={"update_policy": UpdatePolicy.UNSUPPORTED}
+    )
+    plugin_resources = fields.Nested(
+        SchedulerPluginResourcesSchema, metadata={"update_policy": UpdatePolicy.UNSUPPORTED}
+    )
+    events = fields.Nested(
+        SchedulerPluginEventsSchema, required=True, metadata={"update_policy": UpdatePolicy.UNSUPPORTED}
+    )
+    monitoring = fields.Nested(SchedulerPluginMonitoringSchema, metadata={"update_policy": UpdatePolicy.UNSUPPORTED})
+    system_users = fields.Nested(
+        SchedulerPluginUserSchema,
+        many=True,
+        validate=validate.Length(max=SCHEDULER_PLUGIN_MAX_NUMBER_OF_USERS),
+        metadata={"update_policy": UpdatePolicy.UNSUPPORTED, "update_key": "Name"},
+    )
+    tags = fields.Nested(
+        TagSchema, many=True, metadata={"update_policy": UpdatePolicy.UNSUPPORTED, "update_key": "Key"}
+    )
+
+    @post_load
+    def make_resource(self, data, **kwargs):
+        """Generate resource."""
+        return SchedulerPluginDefinition(**data)
+
+
+class SchedulerPluginSettingsSchema(BaseSchema):
+    """Represent the schema of the Scheduling Settings."""
+
+    scheduler_definition = fields.Nested(
+        SchedulerPluginDefinitionSchema, required=True, metadata={"update_policy": UpdatePolicy.UNSUPPORTED}
+    )
+    grant_sudo_privileges = fields.Bool(metadata={"update_policy": UpdatePolicy.UNSUPPORTED})
+    custom_settings = fields.Dict(metadata={"update_policy": UpdatePolicy.COMPUTE_FLEET_STOP})
+
+    @post_load
+    def make_resource(self, data, **kwargs):
+        """Generate resource."""
+        return SchedulerPluginSettings(**data)
+
+    @pre_load
+    def fetch_scheduler_definition(self, data, **kwargs):
+        """Fetch scheduler definition if it is s3 or https url."""
+        original_scheduler_definition = data["SchedulerDefinition"]
+        if isinstance(original_scheduler_definition, str):
+            LOGGER.info("Downloading scheduler plugin definition from %s", original_scheduler_definition)
+            try:
+                if original_scheduler_definition.startswith("s3"):
+                    bucket_parsing_result = parse_bucket_url(original_scheduler_definition)
+                    result = AWSApi.instance().s3.get_object(
+                        bucket_name=bucket_parsing_result["bucket_name"], key=bucket_parsing_result["object_key"]
+                    )
+                    scheduler_definition = result["Body"].read().decode("utf-8")
+                elif original_scheduler_definition.startswith("https"):
+                    try:
+                        with urlopen(original_scheduler_definition) as f:  # nosec nosemgrep
+                            scheduler_definition = f.read().decode("utf-8")
+                    except Exception:
+                        raise ValidationError("The provided URL is invalid or unavailable.")
+                else:
+                    raise ValidationError(
+                        "The provided value for SchedulerDefinition is invalid. "
+                        "You can specify this as an S3 URL, HTTPS URL or as an inline YAML object."
+                    )
+                LOGGER.info("Using the following scheduler plugin definition:\n%s", scheduler_definition)
+                data["SchedulerDefinition"] = yaml.safe_load(scheduler_definition)
+            except YAMLError as e:
+                raise ValidationError(
+                    f"The retrieved SchedulerDefinition ({original_scheduler_definition}) is not a valid YAML."
+                ) from e
+            except Exception as e:
+                raise ValidationError(
+                    f"Error while downloading scheduler definition from {original_scheduler_definition}: {str(e)}"
+                ) from e
+        return data
+
+
 class SchedulingSchema(BaseSchema):
     """Represent the schema of the Scheduling."""
 
     scheduler = fields.Str(
         required=True,
-        validate=validate.OneOf(["slurm", "awsbatch", "custom"]),
+        validate=validate.OneOf(["slurm", "awsbatch"]),
         metadata={"update_policy": UpdatePolicy.UNSUPPORTED},
     )
     # Slurm schema
@@ -1123,7 +1506,6 @@ class SchedulingSchema(BaseSchema):
     slurm_queues = fields.Nested(
         SlurmQueueSchema,
         many=True,
-        validate=validate.Length(max=MAX_NUMBER_OF_QUEUES),
         metadata={"update_policy": UpdatePolicy.COMPUTE_FLEET_STOP, "update_key": "Name"},
     )
     # Awsbatch schema:
@@ -1136,15 +1518,31 @@ class SchedulingSchema(BaseSchema):
     aws_batch_settings = fields.Nested(
         AwsBatchSettingsSchema, metadata={"update_policy": UpdatePolicy.COMPUTE_FLEET_STOP}
     )
+    # Scheduler Plugin
+    scheduler_settings = fields.Nested(
+        SchedulerPluginSettingsSchema, metadata={"update_policy": UpdatePolicy.COMPUTE_FLEET_STOP}
+    )
+    scheduler_queues = fields.Nested(
+        SchedulerPluginQueueSchema,
+        many=True,
+        metadata={"update_policy": UpdatePolicy.COMPUTE_FLEET_STOP, "update_key": "Name"},
+    )
 
     @validates_schema
     def no_coexist_schedulers(self, data, **kwargs):
         """Validate that *_settings and *_queues for different schedulers do not co-exist."""
         scheduler = data.get("scheduler")
-        if self.fields_coexist(data, ["aws_batch_settings", "slurm_settings"], **kwargs):
+        if self.fields_coexist(data, ["aws_batch_settings", "slurm_settings", "scheduler_settings"], **kwargs):
             raise ValidationError("Multiple *Settings sections cannot be specified in the Scheduling section.")
-        if self.fields_coexist(data, ["aws_batch_queues", "slurm_queues"], one_required=True, **kwargs):
-            scheduler_prefix = "AwsBatch" if scheduler == "awsbatch" else scheduler.capitalize()
+        if self.fields_coexist(
+            data, ["aws_batch_queues", "slurm_queues", "scheduler_queues"], one_required=True, **kwargs
+        ):
+            if scheduler == "awsbatch":
+                scheduler_prefix = "AwsBatch"
+            elif scheduler == "plugin":
+                scheduler_prefix = "Scheduler"
+            else:
+                scheduler_prefix = scheduler.capitalize()
             raise ValidationError(f"{scheduler_prefix}Queues section must be specified in the Scheduling section.")
 
     @validates_schema
@@ -1153,6 +1551,7 @@ class SchedulingSchema(BaseSchema):
         for scheduler, settings, queues in [
             ("awsbatch", "aws_batch_settings", "aws_batch_queues"),
             ("slurm", "slurm_settings", "slurm_queues"),
+            ("plugin", "scheduler_settings", "scheduler_queues"),
         ]:
             # Verify the settings section is associated to the right scheduler type
             configured_scheduler = data.get("scheduler")
@@ -1168,8 +1567,8 @@ class SchedulingSchema(BaseSchema):
     @validates_schema
     def same_subnet_in_different_queues(self, data, **kwargs):
         """Validate subnet_ids configured in different queues are the same."""
-        queues = "slurm_queues"
-        if queues in data:
+        if "slurm_queues" in data or "scheduler_queues" in data:
+            queues = "slurm_queues" if "slurm_queues" in data else "scheduler_queues"
 
             def _queue_has_subnet_ids(queue):
                 return queue.networking and queue.networking.subnet_ids
@@ -1184,24 +1583,57 @@ class SchedulingSchema(BaseSchema):
         scheduler = data.get("scheduler")
         if scheduler == "slurm":
             return SlurmScheduling(queues=data.get("slurm_queues"), settings=data.get("slurm_settings", None))
+        if scheduler == "plugin":
+            return SchedulerPluginScheduling(
+                queues=data.get("scheduler_queues"), settings=data.get("scheduler_settings", None)
+            )
         if scheduler == "awsbatch":
             return AwsBatchScheduling(
                 queues=data.get("aws_batch_queues"), settings=data.get("aws_batch_settings", None)
             )
-        # if data.get("custom_queues"):
-        #    return CustomScheduling(**data)
         return None
 
     @pre_dump
     def restore_child(self, data, **kwargs):
         """Restore back the child in the schema, see post_load action."""
         adapted_data = copy.deepcopy(data)
-        scheduler_prefix = "aws_batch" if adapted_data.scheduler == "awsbatch" else adapted_data.scheduler
+        if adapted_data.scheduler == "awsbatch":
+            scheduler_prefix = "aws_batch"
+        elif adapted_data.scheduler == "plugin":
+            scheduler_prefix = "scheduler"
+        else:
+            scheduler_prefix = adapted_data.scheduler
         setattr(adapted_data, f"{scheduler_prefix}_queues", copy.copy(getattr(adapted_data, "queues", None)))
         setattr(adapted_data, f"{scheduler_prefix}_settings", copy.copy(getattr(adapted_data, "settings", None)))
         return adapted_data
 
 
+class DirectoryServiceSchema(BaseSchema):
+    """Represent the schema of the DirectoryService."""
+
+    domain_name = fields.Str(required=True, metadata={"update_policy": UpdatePolicy.COMPUTE_FLEET_STOP})
+    domain_addr = fields.Str(required=True, metadata={"update_policy": UpdatePolicy.COMPUTE_FLEET_STOP})
+    password_secret_arn = fields.Str(
+        required=True,
+        validate=validate.Regexp(r"^arn:.*:secret"),
+        metadata={"update_policy": UpdatePolicy.COMPUTE_FLEET_STOP},
+    )
+    domain_read_only_user = fields.Str(required=True, metadata={"update_policy": UpdatePolicy.COMPUTE_FLEET_STOP})
+    ldap_tls_ca_cert = fields.Str(metadata={"update_policy": UpdatePolicy.COMPUTE_FLEET_STOP})
+    ldap_tls_req_cert = fields.Str(
+        validate=validate.OneOf(["never", "allow", "try", "demand", "hard"]),
+        metadata={"update_policy": UpdatePolicy.COMPUTE_FLEET_STOP},
+    )
+    ldap_access_filter = fields.Str(metadata={"update_policy": UpdatePolicy.COMPUTE_FLEET_STOP})
+    generate_ssh_keys_for_users = fields.Bool(metadata={"update_policy": UpdatePolicy.COMPUTE_FLEET_STOP})
+    additional_sssd_configs = fields.Dict(metadata={"update_policy": UpdatePolicy.COMPUTE_FLEET_STOP})
+
+    @post_load
+    def make_resource(self, data, **kwargs):
+        """Generate resource."""
+        return DirectoryService(**data)
+
+
 class ClusterSchema(BaseSchema):
     """Represent the schema of the Cluster."""
 
@@ -1221,8 +1653,13 @@ class ClusterSchema(BaseSchema):
 
     monitoring = fields.Nested(MonitoringSchema, metadata={"update_policy": UpdatePolicy.SUPPORTED})
     additional_packages = fields.Nested(AdditionalPackagesSchema, metadata={"update_policy": UpdatePolicy.UNSUPPORTED})
-    tags = fields.Nested(TagSchema, many=True, metadata={"update_policy": UpdatePolicy.SUPPORTED, "update_key": "Key"})
+    tags = fields.Nested(
+        TagSchema, many=True, metadata={"update_policy": UpdatePolicy.UNSUPPORTED, "update_key": "Key"}
+    )
     iam = fields.Nested(ClusterIamSchema, metadata={"update_policy": UpdatePolicy.SUPPORTED})
+    directory_service = fields.Nested(
+        DirectoryServiceSchema, metadata={"update_policy": UpdatePolicy.COMPUTE_FLEET_STOP}
+    )
     config_region = fields.Str(data_key="Region", metadata={"update_policy": UpdatePolicy.UNSUPPORTED})
     custom_s3_bucket = fields.Str(metadata={"update_policy": UpdatePolicy.READ_ONLY_RESOURCE_BUCKET})
     additional_resources = fields.Str(metadata={"update_policy": UpdatePolicy.SUPPORTED})
@@ -1238,19 +1675,16 @@ class ClusterSchema(BaseSchema):
         validate_no_reserved_tag(tags)
 
     @validates_schema
-    def no_intel_software_for_batch(self, data, **kwargs):
-        """Ensure IntelSoftware section is not included when AWS Batch is the scheduler."""
+    def no_settings_for_batch(self, data, **kwargs):
+        """Ensure IntelSoftware and DirectoryService section is not included when AWS Batch is the scheduler."""
         scheduling = data.get("scheduling")
-        additional_packages = data.get("additional_packages")
-        if (
-            scheduling
-            and scheduling.scheduler == "awsbatch"
-            and additional_packages
-            and additional_packages.intel_software.intel_hpc_platform
-        ):
-            raise ValidationError(
-                "The use of the IntelSoftware configuration is not supported when using awsbatch as the scheduler."
-            )
+        if scheduling and scheduling.scheduler == "awsbatch":
+            error_message = "The use of the {} configuration is not supported when using awsbatch as the scheduler."
+            additional_packages = data.get("additional_packages")
+            if additional_packages and additional_packages.intel_software.intel_hpc_platform:
+                raise ValidationError(error_message.format("IntelSoftware"))
+            if data.get("directory_service"):
+                raise ValidationError(error_message.format("DirectoryService"))
 
     @post_load(pass_original=True)
     def make_resource(self, data, original_data, **kwargs):
@@ -1260,8 +1694,10 @@ class ClusterSchema(BaseSchema):
             cluster = SlurmClusterConfig(cluster_name=self.cluster_name, **data)
         elif scheduler == "awsbatch":
             cluster = AwsBatchClusterConfig(cluster_name=self.cluster_name, **data)
-        else:  # scheduler == "custom":
-            cluster = BaseClusterConfig(cluster_name=self.cluster_name, **data)  # FIXME Must be ByosCluster
+        elif scheduler == "plugin":
+            cluster = SchedulerPluginClusterConfig(cluster_name=self.cluster_name, **data)
+        else:
+            raise ValidationError(f"Unsupported scheduler {scheduler}.")
 
         cluster.source_config = original_data
         return cluster
