@@ -184,7 +184,7 @@ class CWDashboardConstruct(Construct):
         metric_list = []
         for metric in metrics:
             cloudwatch_metric = cloudwatch.Metric(
-                namespace="AWS/EC2", metric_name=metric, dimensions={"InstanceId": self.head_node_instance.ref}
+                namespace="AWS/EC2", metric_name=metric, dimensions_map={"InstanceId": self.head_node_instance.ref}
             )
             metric_list.append(cloudwatch_metric)
         return metric_list
@@ -206,7 +206,7 @@ class CWDashboardConstruct(Construct):
                     cloudwatch_metric = cloudwatch.Metric(
                         namespace=namespace,
                         metric_name=metric_condition_params.metrics,
-                        dimensions={dimension_vol_name: volume.id},
+                        dimensions_map={dimension_vol_name: volume.id},
                     )
                     metric_list.append(cloudwatch_metric)
 
@@ -224,7 +224,7 @@ class CWDashboardConstruct(Construct):
                     cloudwatch_metric = cloudwatch.Metric(
                         namespace=namespace,
                         metric_name=metric,
-                        dimensions={dimension_name: storage.id},
+                        dimensions_map={dimension_name: storage.id},
                     )
                     metric_list.append(cloudwatch_metric)
             graph_widget = self._generate_graph_widget(metrics_param.title, metric_list)
@@ -326,7 +326,7 @@ class CWDashboardConstruct(Construct):
         widgets_list = self._add_storage_widgets(
             metrics=fsx_metrics,
             storages_list=fsx_volumes_list,
-            namespace="AWS/FSX",
+            namespace="AWS/FSx",
             dimension_name="FileSystemId",
         )
         self.cloudwatch_dashboard.add_widgets(*widgets_list)
@@ -366,7 +366,7 @@ class CWDashboardConstruct(Construct):
                 ],
             ),
             SectionWidgets(
-                "Scheduler''s logs",
+                "Scheduler's logs",
                 [
                     self._new_cw_log_widget(
                         title="slurmctld",
@@ -416,7 +416,7 @@ class CWDashboardConstruct(Construct):
                 ],
             ),
             SectionWidgets(
-                "System''s logs",
+                "System's logs",
                 [
                     self._new_cw_log_widget(
                         title="system-messages",
