@@ -10,6 +10,7 @@
 # limitations under the License.
 import re
 from abc import ABC
+from enum import Enum
 
 from pcluster.aws.aws_api import AWSApi
 from pcluster.aws.common import AWSClientError
@@ -28,6 +29,7 @@ from pcluster.constants import (
 from pcluster.utils import get_installed_version, get_supported_os_for_architecture, get_supported_os_for_scheduler
 from pcluster.validators.common import FailureLevel, Validator
 
+# pylint: disable=C0302
 NAME_MAX_LENGTH = 25
 SHARED_STORAGE_NAME_MAX_LENGTH = 30
 NAME_REGEX = r"^[a-z][a-z0-9\-]*$"
@@ -246,20 +248,16 @@ class NameValidator(Validator):
             self._add_failure(f"It is forbidden to use '{name}' as a name.", FailureLevel.ERROR)
 
 
-class DuplicateInstanceTypeValidator(Validator):
-    """
-    Instance type validator.
+class MaxCountValidator(Validator):
+    """Validate whether the number of resource exceeds the limits."""
 
-    Verify if there are duplicated instance types between compute resources in the same queue.
-    """
+    def _validate(self, resources_length, max_length, resource_name):
 
-    def _validate(self, instance_type_list):
-        duplicated_instance_types = _find_duplicate_params(instance_type_list)
-        if duplicated_instance_types:
+        if resources_length > max_length:
             self._add_failure(
-                "Instance {0} {1} cannot be specified for multiple compute resources in the same queue.".format(
-                    "types" if len(duplicated_instance_types) > 1 else "type",
-                    ", ".join(instance_type for instance_type in duplicated_instance_types),
+                "Invalid number of {resource_name} ({resources_length}) specified. Currently only supports "
+                "up to {max_length} {resource_name}.".format(
+                    resource_name=resource_name, resources_length=resources_length, max_length=max_length
                 ),
                 FailureLevel.ERROR,
             )
@@ -269,9 +267,9 @@ class DuplicateInstanceTypeValidator(Validator):
 
 
 class EfaValidator(Validator):
-    """Check if EFA is supported feature in the given instance type."""
+    """Check if EFA and EFA GDR are supported features in the given instance type."""
 
-    def _validate(self, instance_type, efa_enabled):
+    def _validate(self, instance_type, efa_enabled, gdr_support):
 
         instance_type_supports_efa = AWSApi.instance().ec2.get_instance_type_info(instance_type).is_efa_supported()
         if efa_enabled and not instance_type_supports_efa:
@@ -280,6 +278,8 @@ class EfaValidator(Validator):
             self._add_failure(
                 f"Instance type '{instance_type}' supports EFA, but it is not enabled.", FailureLevel.WARNING
             )
+        if gdr_support and not efa_enabled:
+            self._add_failure("The EFA GDR Support can be used only if EFA is enabled.", FailureLevel.ERROR)
 
 
 class EfaPlacementGroupValidator(Validator):
@@ -322,30 +322,10 @@ class EfaSecurityGroupValidator(Validator):
         for security_group in security_groups:
             try:
                 sec_group = AWSApi.instance().ec2.describe_security_group(security_group)
-                allowed_in = False
-                allowed_out = False
-
                 # check inbound rules
-                for rule in sec_group.get("IpPermissions"):
-                    # UserIdGroupPairs is always of length 1, so grabbing 0th object is ok
-                    if (
-                        rule.get("IpProtocol") == "-1"
-                        and rule.get("UserIdGroupPairs")
-                        and rule.get("UserIdGroupPairs")[0].get("GroupId") == security_group
-                    ):
-                        allowed_in = True
-                        break
-
+                allowed_in = self._all_traffic_allowed(security_group, sec_group.get("IpPermissions"))
                 # check outbound rules
-                for rule in sec_group.get("IpPermissionsEgress"):
-                    if (
-                        rule.get("IpProtocol") == "-1"
-                        and rule.get("UserIdGroupPairs")
-                        and rule.get("UserIdGroupPairs")[0].get("GroupId") == security_group
-                    ):
-                        allowed_out = True
-                        break
-
+                allowed_out = self._all_traffic_allowed(security_group, sec_group.get("IpPermissionsEgress"))
                 if allowed_in and allowed_out:
                     efa_sg_found = True
                     break
@@ -353,17 +333,13 @@ class EfaSecurityGroupValidator(Validator):
                 self._add_failure(str(e), FailureLevel.WARNING)
         return efa_sg_found
 
-
-class EfaGdrValidator(Validator):
-    """Hint that the GdrSupport has no effect."""
-
-    def _validate(self, gdr_support):
-        if gdr_support:
-            self._add_failure(
-                "The GDR Support setting is ignored because EFA enables GDR support by default "
-                "on supported instance type(s).",
-                FailureLevel.INFO,
-            )
+    def _all_traffic_allowed(self, security_group_id, security_group_permission):
+        for rule in security_group_permission:
+            if rule.get("IpProtocol") == "-1" and rule.get("UserIdGroupPairs"):
+                for group in rule.get("UserIdGroupPairs"):
+                    if group.get("GroupId") == security_group_id:
+                        return True
+        return False
 
 
 # --------------- Storage validators --------------- #
@@ -375,7 +351,7 @@ def _check_in_out_access(security_groups_ids, port):
 
     :param security_groups_ids: list of security groups to verify
     :param port: port to verify
-    :return true if
+    :return: True if both in and out access are allowed
     :raise: ClientError if a given security group doesn't exist
     """
     in_out_access = False
@@ -521,7 +497,7 @@ def _find_overlapping_paths(paths_list):
     overlapping_paths = []
     if paths_list:
         for path in paths_list:
-            is_overlapping = any(x for x in paths_list if x != path and x.startswith(path))
+            is_overlapping = any(x for x in paths_list if x != path and x.startswith(path + "/"))
             if is_overlapping:
                 overlapping_paths.append(path)
 
@@ -723,6 +699,49 @@ class DuplicateNameValidator(Validator):
             )
 
 
+class QueuesSecurityGroupOverwriteStatus(Enum):
+    """Define queues security group overwrite status."""
+
+    MIXED = "mixed"
+    CUSTOM = "custom"
+    MANAGED = "managed"
+
+
+class MixedSecurityGroupOverwriteValidator(Validator):
+    """Warn if some nodes are using custom security groups while others are using managed security groups."""
+
+    def _validate(self, head_node_security_groups, queues):
+        compute_security_group_overwrite_status = self._get_queues_security_group_overwrite_status(queues)
+        if (
+            head_node_security_groups
+            and compute_security_group_overwrite_status != QueuesSecurityGroupOverwriteStatus.CUSTOM
+        ) or (
+            head_node_security_groups is None
+            and compute_security_group_overwrite_status != QueuesSecurityGroupOverwriteStatus.MANAGED
+        ):
+            self._add_failure(
+                "Please make sure that all cluster nodes are reachable to each other, "
+                "or consider using additional security groups rather than replacing ParallelCluster security groups.",
+                FailureLevel.WARNING,
+            )
+
+    def _get_queues_security_group_overwrite_status(self, queues):
+        """Check if all queues need managed SG, or use custom SG, or are in mixed condition."""
+        managed = False
+        custom = False
+        for queue in queues:
+            if queue.networking.security_groups:
+                custom = True
+            else:
+                managed = True
+        if custom and managed:
+            return QueuesSecurityGroupOverwriteStatus.MIXED
+        if custom:
+            return QueuesSecurityGroupOverwriteStatus.CUSTOM
+        else:
+            return QueuesSecurityGroupOverwriteStatus.MANAGED
+
+
 # --------------- Instance settings validators --------------- #
 
 
@@ -765,17 +784,17 @@ class _LaunchTemplateValidator(Validator, ABC):
             elif code == "InstanceLimitExceeded":
                 self._add_failure(
                     "You've reached the limit on the number of instances you can run concurrently "
-                    f"for the configured instance type.\n{message}",
+                    f"for the configured instance type. {message}",
                     FailureLevel.ERROR,
                 )
             elif code == "InsufficientInstanceCapacity":
                 self._add_failure(
-                    f"There is not enough capacity to fulfill your request.\n{message}", FailureLevel.ERROR
+                    f"There is not enough capacity to fulfill your request. {message}", FailureLevel.ERROR
                 )
             elif code == "InsufficientFreeAddressesInSubnet":
                 self._add_failure(
                     "The specified subnet does not contain enough free private IP addresses "
-                    f"to fulfill your request.\n{message}",
+                    f"to fulfill your request. {message}",
                     FailureLevel.ERROR,
                 )
             elif code == "InvalidParameterCombination":
@@ -805,7 +824,7 @@ class _LaunchTemplateValidator(Validator, ABC):
             else:
                 self._add_failure(
                     f"Unable to validate configuration parameters for instance type {kwargs['InstanceType']}. "
-                    f"Please double check your cluster configuration.\n{message}",
+                    f"Please double check your cluster configuration. {message}",
                     FailureLevel.ERROR,
                 )
 
@@ -859,7 +878,7 @@ class HeadNodeLaunchTemplateValidator(_LaunchTemplateValidator):
             )
         except Exception as e:
             self._add_failure(
-                f"Unable to validate configuration parameters for the head node.\n{str(e)}", FailureLevel.ERROR
+                f"Unable to validate configuration parameters for the head node. {str(e)}", FailureLevel.ERROR
             )
 
 
@@ -918,7 +937,7 @@ class ComputeResourceLaunchTemplateValidator(_LaunchTemplateValidator):
             )
         except Exception as e:
             self._add_failure(
-                f"Unable to validate configuration parameters for queue {queue.name}.\n{str(e)}", FailureLevel.ERROR
+                f"Unable to validate configuration parameters for queue {queue.name}. {str(e)}", FailureLevel.ERROR
             )
 
     def _test_compute_resource(
